import 'package:pam_flutter/types.dart';

class PamQueueManager {
  bool _isProcessing = false;
  List<ITrackingQueue> _trackingQueue =
      List<ITrackingQueue>.empty(growable: true);
  final Future<void> Function(
    String eventName,
    Map<String, dynamic> payload, {
    bool deleteLoginContactAfterPost,
    bool ignoreContactId,
  })? callback;

  PamQueueManager({
    this.callback,
  });

  void enqueue(
    String eventName, {
    Map<String, dynamic>? payload,
    bool deleteLoginContactAfterPost = false,
    bool ignoreContactId = false,
  }) {
    Map<String, dynamic> newPayload = {};
    if (payload != null) {
      newPayload = payload;
    }

    this._trackingQueue.add(
          ITrackingQueue(
            eventName: eventName,
            payload: newPayload,
            deleteLoginContactAfterPost: deleteLoginContactAfterPost,
            ignoreContactId: ignoreContactId,
          ),
        );

    if (!this._isProcessing) {
      this.nextQueue();
    }
  }

  void nextQueue() {
    if (this._trackingQueue.length > 0) {
      this._isProcessing = true;
      ITrackingQueue task = this._trackingQueue.removeAt(0);
      this.callback?.call(
            task.eventName,
            task.payload,
            deleteLoginContactAfterPost: task.deleteLoginContactAfterPost,
            ignoreContactId: task.ignoreContactId,
          );
    } else {
      this._isProcessing = false;
    }
  }
}
