// ignore_for_file: non_constant_identifier_names

import 'package:flutter/cupertino.dart';
import 'package:pam_flutter/models/consent_message_model.dart';
import 'package:pam_flutter/models/contact_consent_model.dart';
import 'package:pam_flutter/pam_flutter.dart';
import 'package:pam_flutter/repositories/base_repository.dart';
import 'package:pam_flutter/repositories/types.dart';
import 'package:pam_flutter/types.dart';
import 'package:pam_flutter/utils/time_helper.dart';

class ConsentMessageTypes {
  static String get tracking_type => "tracking_type";

  static String get contacting_type => "contacting_type";
}

class ConsentMessageSettingConsentDetails {
  ConsentMessageSettingConsentDetails._();

  static String get terms_and_conditions => "TermsAndConditions";

  static String get analytics_cookies => "AnalyticsCookies";

  static String get email => "Email";

  static String get facebook_messenger => "FacebookMessenger";

  static String get line => "Line";

  static String get marketing_cookies => "MarketingCookies";

  static String get mobile_notification => "MobileNotification";

  static String get necessary_cookies => "NecessaryCookies";

  static String get preferences_cookies => "PreferencesCookies";

  static String get privacy_overview => "PrivacyOverview";

  static String get sms => "SMS";

  static String get social_media_cookies => "SocialMediaCookies";
}

class PDPARepository extends BaseDataRepository<ConsentMessageModel> {
  final BuildContext buildCtx;
  final IRepositoryOptions options;

  PDPARepository({
    required this.buildCtx,
    required this.options,
  }) : super(buildCtx, options);

  @override
  List<ConsentMessageModel> transforms(tss) {
    return ConsentMessageModel.toList(tss);
  }

  @override
  ConsentMessageModel? transform(ts) {
    return ConsentMessageModel.fromJson(ts);
  }

  void allowConsentSetting(String settingName, bool isAllowed) {
    try {
      this.toLoadingStatus();
      ConsentMessageModel model = this.data!;

      switch (settingName) {
        case "TermsAndConditions":
          model.Setting.TermsAndConditions.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "AnalyticsCookies":
          model.Setting.AnalyticsCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "Email":
          model.Setting.Email.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "FacebookMessenger":
          model.Setting.FacebookMessenger.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "Line":
          model.Setting.Line.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "MarketingCookies":
          model.Setting.MarketingCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "MobileNotification":
          model.Setting.MobileNotification.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "NecessaryCookies":
          model.Setting.NecessaryCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "PreferencesCookies":
          model.Setting.PreferencesCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "PrivacyOverview":
          model.Setting.PrivacyOverview.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "SMS":
          model.Setting.SMS.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        case "SocialMediaCookies":
          model.Setting.SocialMediaCookies.IsAllowed = isAllowed;
          super.setInnerData(model.toMap());
          break;
        default:
      }
      this.dataSC.add(this.data);

      this.toLoadedStatus();
    } catch (e) {
      this.toErrorStatus(e);
    }
  }

  void allowAllConsentSetting() {
    try {
      this.toLoadingStatus();
      ConsentMessageModel model = this.data!;

      model.Setting.TermsAndConditions.IsAllowed = true;
      model.Setting.AnalyticsCookies.IsAllowed = true;
      model.Setting.Email.IsAllowed = true;
      model.Setting.FacebookMessenger.IsAllowed = true;
      model.Setting.Line.IsAllowed = true;
      model.Setting.MarketingCookies.IsAllowed = true;
      model.Setting.MobileNotification.IsAllowed = true;
      model.Setting.NecessaryCookies.IsAllowed = true;
      model.Setting.PreferencesCookies.IsAllowed = true;
      model.Setting.PrivacyOverview.IsAllowed = true;
      model.Setting.SMS.IsAllowed = true;
      model.Setting.SocialMediaCookies.IsAllowed = true;

      super.setInnerData(model.toMap());

      this.dataSC.add(this.data);

      this.toLoadedStatus();
    } catch (e) {
      this.toErrorStatus(e);
    }
  }

  Future<IPamResponse?> sendAllowConsentSetting(String id,
      {bool isSubmitTracking = true}) async {
    try {
      this.toLoadingStatus();

      if (this.data?.Type == ConsentMessageTypes.tracking_type) {
        if (isSubmitTracking) {
          Map<String, bool> payload = {};

          if (this.data?.Setting.TermsAndConditions.IsEnabled == true) {
            payload["_allow_terms_and_conditions"] =
                this.data?.Setting.TermsAndConditions.IsAllowed ?? true;
          }
          if (this.data?.Setting.PrivacyOverview.IsEnabled == true) {
            payload["_allow_privacy_overview"] =
                this.data?.Setting.PrivacyOverview.IsAllowed ?? true;
          }
          if (this.data?.Setting.NecessaryCookies.IsEnabled == true) {
            payload["_allow_necessary_cookies"] =
                this.data?.Setting.NecessaryCookies.IsAllowed ?? true;
          }
          if (this.data?.Setting.PreferencesCookies.IsEnabled == true) {
            payload["_allow_preferences_cookies"] =
                this.data?.Setting.PreferencesCookies.IsAllowed ?? true;
          }
          if (this.data?.Setting.AnalyticsCookies.IsEnabled == true) {
            payload["_allow_analytics_cookies"] =
                this.data?.Setting.AnalyticsCookies.IsAllowed ?? true;
          }
          if (this.data?.Setting.MarketingCookies.IsEnabled == true) {
            payload["_allow_marketing_cookies"] =
                this.data?.Setting.MarketingCookies.IsAllowed ?? true;
          }
          if (this.data?.Setting.SocialMediaCookies.IsEnabled == true) {
            payload["_allow_social_media_cookies"] =
                this.data?.Setting.SocialMediaCookies.IsAllowed ?? true;
          }

          this.toLoadedStatus();

          return await Pam.asyncTrackCustomEvent(eventName: "allow_consent", payload: {
            "_consent_message_id": id,
            ...payload,
            "_version": this.data?.Setting.Version,
          });
        }
      } else if (this.data?.Type == ConsentMessageTypes.contacting_type) {
        if (isSubmitTracking) {
          Map<String, bool> payload = {};

          if (this.data?.Setting.TermsAndConditions.IsEnabled == true) {
            payload["_allow_terms_and_conditions"] =
                this.data?.Setting.TermsAndConditions.IsAllowed ?? true;
          }
          if (this.data?.Setting.TermsAndConditions.IsEnabled == true) {
            payload["_allow_privacy_overview"] =
                this.data?.Setting.TermsAndConditions.IsAllowed ?? true;
          }
          if (this.data?.Setting.Email.IsEnabled == true) {
            payload["_allow_email"] =
                this.data?.Setting.Email.IsAllowed ?? true;
          }
          if (this.data?.Setting.SMS.IsEnabled == true) {
            payload["_allow_sms"] = this.data?.Setting.SMS.IsAllowed ?? true;
          }
          if (this.data?.Setting.Line.IsEnabled == true) {
            payload["_allow_line"] = this.data?.Setting.Line.IsAllowed ?? true;
          }
          if (this.data?.Setting.FacebookMessenger.IsEnabled == true) {
            payload["_allow_facebook"] =
                this.data?.Setting.FacebookMessenger.IsAllowed ?? true;
          }
          if (this.data?.Setting.MobileNotification.IsEnabled == true) {
            payload["_allow_mobile_notification"] =
                this.data?.Setting.MobileNotification.IsAllowed ?? true;
          }

          this.toLoadedStatus();

          return await Pam.asyncTrackCustomEvent(eventName: "allow_consent", payload: {
            "_consent_message_id": id,
            ...payload,
            "_version": this.data?.Setting.Version,
          });
        }
      }

      this.toLoadedStatus();
    } catch (e) {
      this.toErrorStatus(e);
      return null;
    }
  }

  Future<IPamResponse?> sendAllowConsentSettingWithConsentSetting(
      String id, ConsentMessageSettingModel? consentMessage) async {
    if (consentMessage == null) {
      return null;
    }

    try {
      this.toLoadingStatus();
      Map<String, bool> payload = {};

      if (consentMessage.TermsAndConditions.IsEnabled == true) {
        payload["_allow_terms_and_conditions"] =
            consentMessage.TermsAndConditions.IsAllowed;
      }
      if (consentMessage.PrivacyOverview.IsEnabled == true) {
        payload["_allow_privacy_overview"] =
            consentMessage.PrivacyOverview.IsAllowed;
      }
      if (consentMessage.NecessaryCookies.IsEnabled == true) {
        payload["_allow_necessary_cookies"] =
            consentMessage.NecessaryCookies.IsAllowed;
      }
      if (consentMessage.PreferencesCookies.IsEnabled == true) {
        payload["_allow_preferences_cookies"] =
            consentMessage.PreferencesCookies.IsAllowed;
      }
      if (consentMessage.AnalyticsCookies.IsEnabled == true) {
        payload["_allow_analytics_cookies"] =
            consentMessage.AnalyticsCookies.IsAllowed;
      }
      if (consentMessage.MarketingCookies.IsEnabled == true) {
        payload["_allow_marketing_cookies"] =
            consentMessage.MarketingCookies.IsAllowed;
      }
      if (consentMessage.SocialMediaCookies.IsEnabled == true) {
        payload["_allow_social_media_cookies"] =
            consentMessage.SocialMediaCookies.IsAllowed;
      }
      if (consentMessage.TermsAndConditions.IsEnabled == true) {
        payload["_allow_terms_and_conditions"] =
            consentMessage.TermsAndConditions.IsAllowed;
      }
      if (consentMessage.TermsAndConditions.IsEnabled == true) {
        payload["_allow_privacy_overview"] =
            consentMessage.TermsAndConditions.IsAllowed;
      }
      if (consentMessage.Email.IsEnabled == true) {
        payload["_allow_email"] = consentMessage.Email.IsAllowed;
      }
      if (consentMessage.SMS.IsEnabled == true) {
        payload["_allow_sms"] = consentMessage.SMS.IsAllowed;
      }
      if (consentMessage.Line.IsEnabled == true) {
        payload["_allow_line"] = consentMessage.Line.IsAllowed;
      }
      if (consentMessage.FacebookMessenger.IsEnabled == true) {
        payload["_allow_facebook"] = consentMessage.FacebookMessenger.IsAllowed;
      }
      if (consentMessage.MobileNotification.IsEnabled == true) {
        payload["_allow_mobile_notification"] =
            consentMessage.MobileNotification.IsAllowed;
      }

      IPamResponse? pamResponse = await Pam.asyncTrackCustomEvent(
        eventName: "allow_consent",
        payload: {
          "_consent_message_id": id,
          ...payload,
          "_version": consentMessage.Version,
        },
      );

      await TimeHelper.sleep();

      this.toLoadedStatus();
      return pamResponse;
    } catch (e) {
      this.toErrorStatus(e);
      return null;
    }
  }

  void parseContactConsent(ContactConsentModel? consentModel) {
    if (consentModel == null) {
      return;
    }

    try {
      this.toLoadingStatus();
      ConsentMessageModel model = this.data!;

      model.Setting.TermsAndConditions.IsAllowed =
          consentModel.TrackingPermission.TermsAndConditions ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.AnalyticsCookies.IsAllowed =
          consentModel.TrackingPermission.AnalyticsCookies ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.Email.IsAllowed = consentModel.ContactingPermission.Email ??
          model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.FacebookMessenger.IsAllowed =
          consentModel.ContactingPermission.Facebook ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.Line.IsAllowed = consentModel.ContactingPermission.Line ??
          model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.MarketingCookies.IsAllowed =
          consentModel.TrackingPermission.MarketingCookies ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.MobileNotification.IsAllowed =
          consentModel.ContactingPermission.MobileNotification ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.NecessaryCookies.IsAllowed =
          consentModel.TrackingPermission.NecessaryCookies ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.PreferencesCookies.IsAllowed =
          consentModel.TrackingPermission.PreferencesCookies ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.PrivacyOverview.IsAllowed =
          consentModel.TrackingPermission.PrivacyOverview ??
              model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.SMS.IsAllowed = consentModel.ContactingPermission.SMS ??
          model.Setting.TermsAndConditions.IsAllowed;
      model.Setting.SocialMediaCookies.IsAllowed =
          consentModel.TrackingPermission.SocialMediaCookies ??
              model.Setting.TermsAndConditions.IsAllowed;
      super.setInnerData(model.toMap());

      this.dataSC.add(this.data);

      this.toLoadedStatus();
    } catch (e) {
      this.toErrorStatus(e);
    }
  }
}
