import 'dart:async';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:pam_flutter/models/consent_message_model.dart';
import 'package:pam_flutter/utils/time_helper.dart';
import 'package:pam_flutter/widgets/commons/switch_button.dart';

// ignore: must_be_immutable
class ConsentItem extends StatefulWidget {
  final String title;
  final ConsentMessageSettingConsentDetailModel? setting;
  final void Function(bool isAllow) onAllowed;
  final String language;
  final ConsentMessageSettingMoreInfoModel? moreInfo;
  final ConsentMessageLanguageModel? acceptText;
  bool? isRequired;
  bool? isOpened;

  ConsentItem({
    required this.title,
    required this.setting,
    required this.onAllowed,
    required this.language,
    required this.moreInfo,
    this.acceptText,
    this.isRequired,
    this.isOpened,
  }) {
    if (this.isRequired == null) {
      this.isRequired = false;
    }
    if (this.isOpened == null) {
      this.isOpened = false;
    }
  }

  @override
  State<StatefulWidget> createState() {
    return ConsentItemState();
  }
}

class ConsentItemState extends State<ConsentItem> {
  late bool _isConsentExpand;
  late StreamController<bool> _isConsentExpandSC;

  late bool _isInnerConsentExpand;
  late StreamController<bool> _isInnerConsentExpandSC;

  @override
  void initState() {
    super.initState();

    _isConsentExpand = false;
    _isInnerConsentExpand = false;
    this._isConsentExpandSC = StreamController<bool>();
    this._isInnerConsentExpandSC = StreamController<bool>();

    this._isConsentExpandSC.add(this._isConsentExpand);
    this._isInnerConsentExpandSC.add(this._isInnerConsentExpand);

    if (widget.isOpened == true) {
      onConsentExpandClicked();
    }
  }

  @override
  void dispose() {
    super.dispose();

    this._isConsentExpandSC.close();
    this._isInnerConsentExpandSC.close();
  }

  onConsentExpandClicked() {
    this._isConsentExpand = !this._isConsentExpand;
    this._isConsentExpandSC.add(this._isConsentExpand);

    if (this._isConsentExpand) {
      this.innerConsentExpandOpened();
    } else {
      this.innerConsentExpandClosed();
    }
  }

  Future<void> innerConsentExpandOpened() async {
    await TimeHelper.sleep(milliseconds: 50);
    this._isInnerConsentExpand = !this._isInnerConsentExpand;
    this._isInnerConsentExpandSC.add(this._isInnerConsentExpand);
  }

  Future<void> innerConsentExpandClosed() async {
    await TimeHelper.sleep(milliseconds: 50);
    this._isInnerConsentExpand = !this._isInnerConsentExpand;
    this._isInnerConsentExpandSC.add(this._isInnerConsentExpand);
  }

  String getLanguage(ConsentMessageLanguageModel? message) {
    if (message == null) {
      return "";
    }

    switch (widget.language) {
      case "th":
        return message.TH;
      case "en":
        return message.EN;
      default:
        return message.TH;
    }
  }

  @override
  Widget build(BuildContext context) {
    if (widget.setting?.IsEnabled == false) {
      return Container();
    }

    return Container(
      child: StreamBuilder<bool>(
          stream: this._isConsentExpandSC.stream,
          builder: (context, snapshot) {
            return Column(
              children: [
                GestureDetector(
                  onTap: () {
                    this.onConsentExpandClicked();
                  },
                  child: Container(
                    decoration: BoxDecoration(
                      color: Colors.white,
                      border: Border(
                        bottom: BorderSide(
                          color: Colors.grey,
                        ),
                      ),
                    ),
                    padding: EdgeInsets.only(
                      top: 20,
                      bottom: 20,
                      left: 16,
                      right: 16,
                    ),
                    child: Row(
                      children: [
                        Container(
                          margin: EdgeInsets.only(right: 12),
                          child: Icon(
                            Icons.check_rounded,
                            color: widget.isRequired!
                                ? Colors.black
                                : (widget.setting?.IsAllowed ?? false
                                ? Colors.green
                                : Colors.grey.shade300),
                            size: 18,
                          ),
                        ),
                        Expanded(
                          child: Container(
                            child: Text(
                              widget.title,
                              style: TextStyle(
                                fontSize: 18,
                                fontWeight: FontWeight.bold,
                              ),
                            ),
                          ),
                        ),
                        Container(
                          margin: EdgeInsets.only(right: 12),
                          child: Icon(
                            this._isConsentExpand
                                ? Icons.keyboard_arrow_down
                                : Icons.keyboard_arrow_up,
                            color: Colors.grey,
                            size: 18,
                          ),
                        ),
                      ],
                    ),
                  ),
                ),
                AnimatedOpacity(
                  duration: Duration(milliseconds: 100),
                  opacity: snapshot.data == true ? 1 : 0,
                  child: StreamBuilder<bool>(
                    stream: this._isInnerConsentExpandSC.stream,
                    builder: (context, innerSnapshot) {
                      return AnimatedContainer(
                        duration: Duration(milliseconds: 100),
                        padding: EdgeInsets.all(16),
                        height: snapshot.data == true ? null : 0,
                        child: Column(
                          children: [
                            Container(
                              child: Text(
                                innerSnapshot.data == true
                                    ? this.getLanguage(
                                    widget.setting?.BriefDescription)
                                    : "",
                                // maxLines: 14,
                                // overflow: TextOverflow.ellipsis,
                              ),
                            ),
                            innerSnapshot.data == true
                                ? Container(
                              margin: EdgeInsets.only(top: 16),
                              child: Row(
                                mainAxisAlignment:
                                MainAxisAlignment.spaceBetween,
                                children: [
                                  Container(
                                    child: Text(
                                      widget.moreInfo != null ? getLanguage(widget.moreInfo!.DisplayText) : "Full Version",
                                      style: TextStyle(
                                        color: Colors.grey.shade600,
                                        decoration:
                                        TextDecoration.underline,
                                      ),
                                    ),
                                  ),
                                  if (!widget.isRequired!)
                                    Container(
                                      child: SwitchButton(
                                        title: "Accept",
                                        value: widget
                                            .setting?.IsAllowed ??
                                            false,
                                        onChanged: (v) {
                                          widget.onAllowed(v);
                                        },
                                        buttonColor: Colors.white,
                                        backgroundColor: Colors.green,
                                      ),
                                    ),
                                ],
                              ),
                            )
                                : Container(),
                          ],
                        ),
                      );
                    },
                  ),
                ),
              ],
            );
          }
      ),
    );
  }
}
